use anyhow::Result;
use chrono::Local;
use rusqlite::Connection;

use crate::models::work_model::*;

pub(crate) struct SqliteService {
    conn: Connection,
}
impl Default for SqliteService {
    fn default() -> Self {
        //TODO: fix cant use path to create service
        return match Self::new(".data.db".to_string()) {
            Ok(service) => service,
            Err(err) => panic!("{}", err.to_string())
        };
    }
}

impl SqliteService {
    pub fn new(conn_str: String) -> Result<Self> {
        let conn = Connection::open(conn_str)?;
        let this = Self { conn };
        this.create_table()?;
        return Ok(this);
    }

    pub fn create_table(&self) -> Result<()> {
        self.conn.execute(
            "CREATE TABLE IF NOT EXISTS write_results (
                id INTEGER PRIMARY KEY,
                line_number INTEGER NOT NULL,
                imei TEXT NOT NULL,
                sn TEXT NOT NULL,
                write_date TEXT NOT NULL
            )",
            [],
        )?;
        Ok(())
    }

    pub fn insert(&self, work_model: WorkModel) -> Result<()> {
        //TODO: How to insert line_number with i32?
        self.conn.execute(
            "INSERT INTO write_results (imei, line_number, sn, write_date) VALUES (?1, ?2, ?3, ?4)",
            &[&work_model.imei, &work_model.line_number, &work_model.sn, &Local::now().format("%Y-%m-%d %H:%M:%S").to_string()],
            )?;
        Ok(())
    }

    pub fn check_sn_has_insert(&self, sn: &str) -> Result<bool> {
        let mut stmt = self.conn.prepare("SELECT sn FROM write_results WHERE sn = ?1")?;
        let mut rows = stmt.query(&[&sn])?;
        Ok(rows.next()?.is_some())
    }
    pub fn check_imei_has_insert(&self, imei: &str) -> Result<bool> {
        let mut stmt = self.conn.prepare("SELECT imei FROM write_results WHERE imei = ?1")?;
        let mut rows = stmt.query(&[&imei])?;
        Ok(rows.next()?.is_some())
    }
}


#[test]
fn create_sql_service_test() {
    let service = SqliteService::default();
}
